from keras.layers import Conv2D
from keras.layers import MaxPooling2D
from keras.layers import Flatten
from keras.layers import Dense
from keras.models import Sequential
from keras.preprocessing.image import ImageDataGenerator
from keras.preprocessing import image
import numpy as np


def main():
    # Define the model
    classifier = Sequential()
    classifier.add(Conv2D(32, (3, 3), input_shape=(64, 64, 3), activation='relu'))
    classifier.add(MaxPooling2D(pool_size=(2, 2)))
    classifier.add(Flatten())
    classifier.add(Dense(units=128, activation='relu'))
    classifier.add(Dense(units=1, activation='sigmoid'))
    classifier.compile(optimizer='adam', loss='binary_crossentropy', metrics=['accuracy'])

    # Train the model
    train_datagen = ImageDataGenerator(
        rescale=1./255,
        shear_range=0.2,
        zoom_range=0.2,
        horizontal_flip=True)
    test_datagen = ImageDataGenerator(rescale = 1./255)
    training_set = train_datagen.flow_from_directory(
        'CNN_Data/training_set',
        target_size=(64, 64),
        batch_size=32,
        class_mode='binary')
    test_set = test_datagen.flow_from_directory(
        'CNN_Data/test_set',
        target_size=(64, 64),
        batch_size=32,
        class_mode='binary')
    classifier.fit_generator(
        training_set,
        steps_per_epoch=8000,
        epochs=25,
        validation_data=test_set,
        validation_steps=2000)

    # Save the model
    model_yaml = classifier.to_yaml()
    with open("model.yaml", "w") as yaml_file:
        yaml_file.write(model_yaml)
    # serialize weights to HDF5
    classifier.save_weights("model.h5")
    print("Saved model to disk")


if __name__ == "__main__":
    main()



import argparse
import os.path
from keras.models import model_from_yaml
from keras.preprocessing import image
import numpy as np
from transcendental import config


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error('The file {} does not exist!'.format(arg))
    return arg


def argparser():
    parser = argparse.ArgumentParser(description='Recognize cat and dogs')
    parser.add_argument('image', help='an image to analyze',
                        type=lambda x: is_valid_file(parser, x))
    return parser.parse_args()


def main():
    args = argparser()
    # load YAML and create model
    yaml_file = open('{}/model.yaml'.format(config.MODELS), 'r')
    loaded_model_yaml = yaml_file.read()
    yaml_file.close()
    classifier = model_from_yaml(loaded_model_yaml)
    # load weights into new model
    classifier.load_weights("{}/model.h5".format(config.MODELS))
    print("Loaded model from disk")

    # Predict with the model
    test_image = image.load_img(args.image, target_size=(64, 64))
    test_image = image.img_to_array(test_image)
    test_image = np.expand_dims(test_image, axis=0)
    result = classifier.predict(test_image)
    prediction = "unknown"
    if result[0][0] == 1:
        prediction = 'dog'
    else:
        prediction = 'cat'
    print("It's a {}".format(prediction))


if __name__ == "__main__":
    main()
